@@ -33,7 +33,6 @@
 #include <glib/gi18n.h>
 
 #include "applet.h"
-#include "menu.h"
 #include "panel-config-global.h"
 #include "panel-gconf.h"
 #include "panel-profile.h"
@@ -45,9 +44,17 @@
 #include "panel-run-dialog.h"
 #include "panel-a11y.h"
 #include "panel-lockdown.h"
+#include "panel-logout.h"
+#include "panel-compatibility.h"
+#include "panel-gdm.h"
+#include "panel-icon-names.h"
+
+G_DEFINE_TYPE (PanelActionButton, panel_action_button, BUTTON_TYPE_WIDGET);
 
 #define PANEL_ACTION_BUTTON_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), PANEL_TYPE_ACTION_BUTTON, PanelActionButtonPrivate))
 
+#define LOGOUT_PROMPT_KEY "/apps/gnome-session/options/logout_prompt"
+
 enum {
 	PROP_0,
 	PROP_ACTION_TYPE,
@@ -63,17 +70,17 @@ struct _PanelActionButtonPrivate {
 	guint                  dnd_enabled : 1;
 };
 
-static GObjectClass *parent_class;
-
 static GConfEnumStringPair panel_action_type_map [] = {
 	{ PANEL_ACTION_NONE,           "none"           },
 	{ PANEL_ACTION_LOCK,           "lock"           },
 	{ PANEL_ACTION_LOGOUT,         "logout"         },
 	{ PANEL_ACTION_RUN,            "run"            },
 	{ PANEL_ACTION_SEARCH,         "search"         },
-	{ PANEL_ACTION_SCREENSHOT,     "screenshot"     },
 	{ PANEL_ACTION_FORCE_QUIT,     "force-quit"     },
 	{ PANEL_ACTION_CONNECT_SERVER, "connect-server" },
+	{ PANEL_ACTION_SHUTDOWN,       "shutdown"       },
+	/* compatibility with GNOME < 2.13.90 */
+	{ PANEL_ACTION_SCREENSHOT,     "screenshot"     },
 	{ 0,                           NULL             },
 };
 
@@ -125,18 +132,6 @@ panel_action_lock_setup_menu (PanelActionButton *button)
 				   _("_Lock Screen"),
 				   screensaver_enabled);
 
-	panel_applet_add_callback (button->priv->info,
-				   "exit",
-				   NULL,
-				   _("_Kill Screensaver Daemon"),
-				   screensaver_properties_enabled);
-
-	panel_applet_add_callback (button->priv->info,
-				   "restart",
-				   NULL,
-				   _("Restart _Screensaver Daemon"),
-				   screensaver_properties_enabled);
-
 	panel_applet_add_callback (button->priv->info,
 				   "prefs",
 				   GTK_STOCK_PROPERTIES,
@@ -160,16 +155,36 @@ panel_action_lock_invoke_menu (PanelActionButton *button,
 static void
 panel_action_logout (GtkWidget *widget)
 {
-	static int recursion_guard = 0;
+	gboolean not_prompt;
 
-	if (recursion_guard)
-		return;
+	not_prompt = gconf_client_get_bool (panel_gconf_get_client (),
+					    LOGOUT_PROMPT_KEY, NULL);
+	/* this avoids handling errors from gconf since prompting is
+	 * safer */
+	not_prompt = !not_prompt;
 
-	recursion_guard++;
+	if (not_prompt)
+		panel_session_request_logout ();
+	else
+		panel_logout_new (PANEL_LOGOUT_DIALOG_LOGOUT,
+				  gtk_widget_get_screen (widget),
+				  gtk_get_current_event_time ());
+}
 
-	panel_session_request_logout ();
+static void
+panel_action_shutdown (GtkWidget *widget)
+{
+	panel_logout_new (PANEL_LOGOUT_DIALOG_SHUTDOWN,
+			  gtk_widget_get_screen (widget),
+			  gtk_get_current_event_time ());
+}
 
-	recursion_guard--;
+static gboolean
+panel_action_shutdown_reboot_is_disabled (void)
+{
+	return (panel_lockdown_get_disable_log_out() ||
+	(!gdm_supports_logout_action (GDM_LOGOUT_ACTION_SHUTDOWN) &&
+	 !gdm_supports_logout_action (GDM_LOGOUT_ACTION_REBOOT)));
 }
 
 /* Run Application
@@ -177,7 +192,8 @@ panel_action_logout (GtkWidget *widget)
 static void
 panel_action_run_program (GtkWidget *widget)
 {
-	panel_run_dialog_present (gtk_widget_get_screen (widget), gtk_get_current_event_time ());
+	panel_run_dialog_present (gtk_widget_get_screen (widget),
+				  gtk_get_current_event_time ());
 }
 
 /* Search For Files
@@ -185,66 +201,22 @@ panel_action_run_program (GtkWidget *widget)
 static void
 panel_action_search (GtkWidget *widget)
 {
-	GnomeDesktopItem *ditem;
 	GdkScreen *screen;
 	GError    *error = NULL;
-	char      *argv[2] = {"gnome-search-tool", NULL};
-	char      *file;
 
 	screen = gtk_widget_get_screen (widget);
-	file = panel_lookup_in_data_dirs ("applications/gnome-search-tool.desktop");
-
-	if (file) {
-		ditem = gnome_desktop_item_new_from_file (file, 0, &error);
-		if (ditem) {
-			g_assert (error == NULL);	
-			panel_ditem_launch (ditem, NULL, 0, screen, &error);
-			gnome_desktop_item_unref (ditem);
-		}
-		if (error) {
-			panel_error_dialog (screen,
-			                    "cannot_exec_gnome-search-tool", TRUE,
-			                    _("Cannot execute '%s'"),
-			                    "%s",
-			                    "gnome-search-tool",
-			                    error->message);
-			g_error_free (error);
-		}
-		g_free (file);
-	}
-	else if (!gdk_spawn_on_screen (screen, NULL, argv, NULL,
-				  G_SPAWN_SEARCH_PATH,
-				  NULL, NULL, NULL, &error)) {
-		panel_error_dialog (screen,
+	panel_launch_desktop_file ("gnome-search-tool.desktop",
+				   "gnome-search-tool",
+				   screen,
+				   &error);
+	if (error) {
+		char *primary;
+		primary = g_strdup_printf (_("Could not execute '%s'"),
+					   "gnome-search-tool");
+		panel_error_dialog (NULL, screen,
 				    "cannot_exec_gnome-search-tool", TRUE,
-				    _("Cannot execute '%s'"),
-				    "%s",
-				    "gnome-search-tool",
-				    error->message);
-		g_error_free (error);
-	}
-}
-
-/* Take Screenshot
- */
-static void
-panel_action_screenshot (GtkWidget *widget)
-{
-	GdkScreen *screen;
-	GError    *error = NULL;
-	char      *argv [2] = {"gnome-screenshot", NULL};
-
-	screen = gtk_widget_get_screen (widget);
-
-	if (!gdk_spawn_on_screen (screen, NULL, argv, NULL,
-				  G_SPAWN_SEARCH_PATH,
-				  NULL, NULL, NULL, &error)) {
-		panel_error_dialog (screen,
-				    "cannot_exec_gnome-screenshot", TRUE,
-				    _("Cannot execute '%s'"),
-				    "%s",
-				    "gnome-screenshot",
-				    error->message);
+				    primary, error->message);
+		g_free (primary);
 		g_error_free (error);
 	}
 }
@@ -254,7 +226,8 @@ panel_action_screenshot (GtkWidget *widget)
 static void
 panel_action_force_quit (GtkWidget *widget)
 {
-	panel_force_quit (gtk_widget_get_screen (widget));
+	panel_force_quit (gtk_widget_get_screen (widget),
+			  gtk_get_current_event_time ());
 }
 
 /* Connect Server
@@ -274,11 +247,11 @@ panel_action_connect_server (GtkWidget *widget)
 	if (error) {
 		GtkWidget *error_dialog;
 
-		error_dialog = panel_error_dialog (screen,
+		error_dialog = panel_error_dialog (NULL, screen,
 						   "cannot_connect_server",
 						   TRUE,
-						   _("Cannot connect to server"),
-						   "%s", error->message);
+						   _("Could not connect to server"),
+						   error->message);
 		g_clear_error (&error);
 	}
 }
@@ -307,7 +280,7 @@ static PanelAction actions [] = {
 	},
 	{
 		PANEL_ACTION_LOCK,
-		"gnome-lockscreen",
+		PANEL_ICON_LOCKSCREEN,
 		N_("Lock Screen"),
 		N_("Protect your computer from unauthorized use"),
 		"gospanel-21",
@@ -319,9 +292,12 @@ static PanelAction actions [] = {
 	},
 	{
 		PANEL_ACTION_LOGOUT,
-		"gnome-logout",
-		N_("Log Out"),
-		N_("Log out of this session to log in as a different user or to shut down the computer"),
+		PANEL_ICON_LOGOUT,
+		/* when changing one of those two strings, don't forget to
+		 * update the ones in panel-menu-items.c (look for
+		 * panel:showusername|1) */
+		N_("Log Out..."),
+		N_("Log out of this session to log in as a different user"),
 		"gospanel-20",
 		"ACTION:logout:NEW",
 		panel_action_logout, NULL, NULL,
@@ -329,9 +305,9 @@ static PanelAction actions [] = {
 	},
 	{
 		PANEL_ACTION_RUN,
-		PANEL_RUN_ICON,
+		PANEL_ICON_RUN,
 		N_("Run Application..."),
-		N_("Run an Application by entering a command"),
+		N_("Run an application by typing a command or choosing from a list"),
 		"gospanel-555",
 		"ACTION:run:NEW",
 		panel_action_run_program, NULL, NULL,
@@ -339,25 +315,16 @@ static PanelAction actions [] = {
 	},
 	{
 		PANEL_ACTION_SEARCH,
-		"gnome-searchtool",
+		PANEL_ICON_SEARCHTOOL,
 		N_("Search for Files..."),
-		N_("Find files, folders, and documents on your computer"),
+		N_("Locate documents and folders on this computer by name or content"),
 		"gospanel-554",
 		"ACTION:search:NEW",
 		panel_action_search, NULL, NULL, NULL
 	},
-	{
-		PANEL_ACTION_SCREENSHOT,
-		"applets-screenshooter",
-		N_("Take Screenshot..."),
-		N_("Take a screenshot of your desktop"),
-		"gospanel-553",
-		"ACTION:screenshot:NEW",
-		panel_action_screenshot, NULL, NULL, NULL
-	},
 	{
 		PANEL_ACTION_FORCE_QUIT,
-		PANEL_FORCE_QUIT_ICON,
+		PANEL_ICON_FORCE_QUIT,
 		N_("Force Quit"),
 		N_("Force a misbehaving application to quit"),
 		"gospanel-563",
@@ -367,20 +334,47 @@ static PanelAction actions [] = {
 	},
 	{
 		PANEL_ACTION_CONNECT_SERVER,
-		"gnome-globe", //FIXME icon
+		PANEL_ICON_REMOTE, //FIXME icon
 		N_("Connect to Server..."),
-		N_("Connect to a remote server"), //FIXME
+		N_("Connect to a remote computer or shared disk"),
 		"gospanel-563", //FIXME
 		"ACTION:connect-server:NEW",
 		panel_action_connect_server, NULL, NULL, NULL
+	},
+	{
+		PANEL_ACTION_SHUTDOWN,
+		PANEL_ICON_SHUTDOWN,
+		N_("Shut Down..."),
+		N_("Shut down the computer"),
+		"gospanel-20",
+		"ACTION:shutdown:NEW",
+		panel_action_shutdown, NULL, NULL,
+		panel_action_shutdown_reboot_is_disabled
+	},
+	/* deprecated actions */
+	{
+		PANEL_ACTION_SCREENSHOT,
+		NULL, NULL, NULL, NULL, NULL,
+		NULL, NULL, NULL, NULL
 	}
 };
 
+static gboolean
+panel_action_get_is_deprecated (PanelActionButtonType type)
+{
+	g_return_val_if_fail (type > PANEL_ACTION_NONE && type < PANEL_ACTION_LAST, FALSE);
+
+	return (type >= PANEL_ACTION_SCREENSHOT);
+}
+
 gboolean
 panel_action_get_is_disabled (PanelActionButtonType type)
 {
 	g_return_val_if_fail (type > PANEL_ACTION_NONE && type < PANEL_ACTION_LAST, FALSE);
 
+	if (panel_action_get_is_deprecated (type))
+		return TRUE;
+
 	if (actions [type].is_disabled)
 		return actions [type].is_disabled ();
 
@@ -452,7 +446,7 @@ panel_action_button_finalize (GObject *object)
 				    button->priv->gconf_notify);
 	button->priv->gconf_notify = 0;
 
-	parent_class->finalize (object);
+	G_OBJECT_CLASS (panel_action_button_parent_class)->finalize (object);
 }
 
 static void
@@ -564,8 +558,6 @@ panel_action_button_class_init (PanelActionButtonClass *klass)
 	GtkWidgetClass *widget_class  = (GtkWidgetClass *) klass;
 	GtkButtonClass *button_class  = (GtkButtonClass *) klass;
 
-	parent_class = g_type_class_peek_parent (klass);
-
 	gobject_class->finalize     = panel_action_button_finalize;
 	gobject_class->get_property = panel_action_button_get_property;
 	gobject_class->set_property = panel_action_button_set_property;
@@ -597,8 +589,7 @@ panel_action_button_class_init (PanelActionButtonClass *klass)
 }
 
 static void
-panel_action_button_instance_init (PanelActionButton      *button,
-				   PanelActionButtonClass *klass)
+panel_action_button_init (PanelActionButton *button)
 {
 	button->priv = PANEL_ACTION_BUTTON_GET_PRIVATE (button);
 
@@ -609,38 +600,15 @@ panel_action_button_instance_init (PanelActionButton      *button,
 	button->priv->dnd_enabled  = FALSE;
 }
 
-GType
-panel_action_button_get_type (void)
-{
-	static GType type = 0;
-
-	if (!type) {
-		static const GTypeInfo info = {
-			sizeof (PanelActionButtonClass),
-			NULL,
-			NULL,
-			(GClassInitFunc) panel_action_button_class_init,
-			NULL,
-			NULL,
-			sizeof (PanelActionButton),
-			0,
-			(GInstanceInitFunc) panel_action_button_instance_init,
-			NULL
-		};
-
-		type = g_type_register_static (
-				BUTTON_TYPE_WIDGET, "PanelActionButton", &info, 0);
-	}
-
-	return type;
-}
-
 void
 panel_action_button_set_type (PanelActionButton     *button,
 			      PanelActionButtonType  type)
 {
 	g_return_if_fail (type > PANEL_ACTION_NONE && type < PANEL_ACTION_LAST);
 
+	if (panel_action_get_is_deprecated (type))
+		return;
+
 	if (type == button->priv->type)
 		return;
 
@@ -649,8 +617,8 @@ panel_action_button_set_type (PanelActionButton     *button,
 	if (actions [type].icon_name != NULL)
 		button_widget_set_icon_name (BUTTON_WIDGET (button), actions [type].icon_name);
 
-	gtk_tooltips_set_tip (panel_tooltips, GTK_WIDGET (button),
-			      _(actions [type].tooltip), NULL);
+	panel_util_set_tooltip_text (GTK_WIDGET (button),
+				     _(actions [type].tooltip));
 	panel_a11y_set_atk_name_desc (GTK_WIDGET (button), _(actions [type].tooltip), NULL);
 
 	panel_action_button_update_sensitivity (button);
@@ -821,7 +789,13 @@ panel_action_button_load_from_gconf (PanelWidget *panel,
 
 	g_free (action_type);
 
-	panel_action_button_load (type, panel, locked, position, exactpos, id, FALSE);
+	/* compatibility: migrate from GNOME < 2.13.90 */
+	if (type == PANEL_ACTION_SCREENSHOT)
+		panel_compatiblity_migrate_screenshot_action (panel_gconf_get_client (),
+							      id);
+	else
+		panel_action_button_load (type, panel, locked,
+					  position, exactpos, id, FALSE);
 }
 
 void
@@ -880,6 +854,9 @@ panel_action_button_load_from_drag (PanelToplevel *toplevel,
 
 	g_return_val_if_fail (type > PANEL_ACTION_NONE && type < PANEL_ACTION_LAST, FALSE);
 
+	if (panel_action_get_is_deprecated (type))
+		return retval;
+
 	if (strcmp (elements [2], "NEW")) {
 		*old_applet_idx = strtol (elements [2], NULL, 10);
 		retval = TRUE; /* Remove the old applet */
@@ -916,8 +893,8 @@ panel_action_button_set_dnd_enabled (PanelActionButton *button,
 				     dnd_targets, 1,
 				     GDK_ACTION_COPY | GDK_ACTION_MOVE);
 		if (actions [button->priv->type].icon_name != NULL)
-		gtk_drag_source_set_icon_name (GTK_WIDGET (button),
-					       actions [button->priv->type].icon_name);
+			gtk_drag_source_set_icon_name (GTK_WIDGET (button),
+						       actions [button->priv->type].icon_name);
 		GTK_WIDGET_SET_FLAGS (button, GTK_NO_WINDOW);
 	} else
 		gtk_drag_source_unset (GTK_WIDGET (button));
