@@ -35,6 +35,7 @@
 #include "panel-recent.h"
 #include "panel-stock-icons.h"
 #include "panel-multiscreen.h"
+#include "panel-icon-names.h"
 
 static gboolean
 show_uri (const char *uri, const char *mime_type, GdkScreen *screen,
@@ -161,11 +162,11 @@ recent_documents_clear_cb (GtkMenuItem      *menuitem,
 
 	clear_recent_dialog = gtk_message_dialog_new (NULL,
 						      0 /* flags */,
-						      GTK_MESSAGE_QUESTION,
+						      GTK_MESSAGE_WARNING,
 						      GTK_BUTTONS_NONE,
 						      _("Clear the Recent Documents list?"));
 	gtk_message_dialog_format_secondary_text (GTK_MESSAGE_DIALOG (clear_recent_dialog),
-						  _("If you clear the Recent Documents list, you clear the following:\n\n"
+						  _("If you clear the Recent Documents list, you clear the following:\n"
 						    "\342\200\242 All items from the Places \342\206\222 Recent Documents menu item.\n"
 						    "\342\200\242 All items from the recent documents list in all applications."));
 
@@ -176,9 +177,13 @@ recent_documents_clear_cb (GtkMenuItem      *menuitem,
 
 	gtk_container_set_border_width (GTK_CONTAINER (clear_recent_dialog), 6);
 
-	gtk_window_set_title (GTK_WINDOW (clear_recent_dialog), "");
+	gtk_window_set_title (GTK_WINDOW (clear_recent_dialog),
+			      _("Clear Recent Documents"));
 
-	gtk_dialog_set_default_response (GTK_DIALOG (clear_recent_dialog), GTK_RESPONSE_ACCEPT);
+	gtk_dialog_set_default_response (GTK_DIALOG (clear_recent_dialog),
+					 GTK_RESPONSE_ACCEPT);
+	gtk_window_set_skip_taskbar_hint (GTK_WINDOW (clear_recent_dialog),
+					  FALSE);
 
 	g_signal_connect (clear_recent_dialog, "response",
 			  G_CALLBACK (clear_dialog_response), manager);
@@ -206,10 +211,9 @@ panel_recent_append_documents_menu (GtkWidget        *top_menu,
 	menu_item = gtk_image_menu_item_new ();
 	setup_menu_item_with_icon (menu_item,
 				   panel_menu_icon_get_size (),
+				   PANEL_ICON_RECENT,
 				   NULL,
-				   GTK_STOCK_OPEN,
-				   _("Recent Documents"),
-				   TRUE);
+				   _("Recent Documents"));
 	recent_menu = gtk_recent_chooser_menu_new_for_manager (manager);
 	gtk_menu_item_set_submenu (GTK_MENU_ITEM (menu_item), recent_menu);
 
@@ -237,6 +241,7 @@ panel_recent_append_documents_menu (GtkWidget        *top_menu,
 				 G_CALLBACK (panel_recent_manager_changed_cb),
 				 menu_item, 0);
 
+	size = 0;
 	g_object_get (manager, "size", &size, NULL);
 	gtk_widget_set_sensitive (menu_item, size > 0);
 
@@ -247,12 +252,9 @@ panel_recent_append_documents_menu (GtkWidget        *top_menu,
 				   panel_menu_icon_get_size (),
 				   NULL,
 				   GTK_STOCK_CLEAR,
-				   _("Clear Recent Documents"),
-				   TRUE);
-	gtk_tooltips_set_tip (panel_tooltips,
-			      menu_item,
-			      _("Clear all items from the recent documents list"),
-			      NULL);
+				   _("Clear Recent Documents"));
+	panel_util_set_tooltip_text (menu_item,
+				     _("Clear all items from the recent documents list"));
 	gtk_menu_shell_append (GTK_MENU_SHELL (recent_menu), menu_item);
 
 	g_signal_connect (menu_item, "activate",
